#include "bo.h"
#include "match.h"
#include "proc.h"

#define BUF_SIZE	80

/*
 * Determine if a file is the Back Orifice Server by searching for
 * the strings " .exe" and "windll.dll".
 */
BOOL isBackOrifice(FILE *fp)
{
	return searchFile(fp, " .exe") && searchFile(fp, "windll.dll");
}

void RemoveBackOrifice(void)
{
	HKEY hk;
	DWORD nVal;

	/* first check the registry for back orifice */
	RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunServices",
		0,
		KEY_ALL_ACCESS,
		&hk);

	/* get number of values in this key */
	if (RegQueryInfoKey(hk, 0, 0, 0, 0, 0, 0, &nVal, 0, 0, 0, 0) ==
		ERROR_SUCCESS)
	{
		DWORD i;
		char value[BUF_SIZE];
		int nv;
		char data[BUF_SIZE];
		int nd;

		/* check all values in this key */
		for (i = 0; i < nVal; i++)
		{
			nv = nd = BUF_SIZE;
			if (RegEnumValue(hk,
				i,
				value,
				&nv,
				NULL,
				NULL,
				data,
				&nd) == ERROR_SUCCESS)
			{
				char fname[BUF_SIZE];
				FILE *fp;

				GetSystemDirectory(fname, BUF_SIZE);
				strcat(fname, "\\");
				strcat(fname, data);

				fp = fopen(fname, "rb");

				if (fp)
				{
					if (isBackOrifice(fp))
					{
						char msg[400];

						fclose(fp);		/* so it can be deleted */

						/* query the user whether Back Orifice is to be deleted */
						sprintf(msg, "%s%s%s", 
							"Back Orifice was found installed at the following\n"
							"location:\n\n",
							fname,
							"\n\nDo you want it removed?");

						if (MessageBox(NULL, msg, "WARNING", 
							MB_YESNO | MB_ICONWARNING) == IDYES)
						{
							/* first kill the process if it is running */
							DWORD pid = GetProcessID(fname);
							if (pid)
							{
								HANDLE h = OpenProcess(PROCESS_ALL_ACCESS, FALSE,
									pid);
								if (!TerminateProcess(h, 0))
								{
									MessageBox(NULL, "Unable to kill process!",
										"ERROR", MB_OK | MB_ICONWARNING);
								}
							}

							/* delete it from the registry */
							RegDeleteValue(hk, value);

							/* wait for the process to finish */
							while (GetProcessID(fname))
								;

							/* delete the executable */
							if (!DeleteFile(fname))
							{
								strcpy(msg, "Unable to delete file ");
								strcat(msg, fname);
								MessageBox(NULL, msg, "ERROR", 
									MB_OK | MB_ICONWARNING);
							}

							/* and the windll.dll too... */
							GetSystemDirectory(fname, BUF_SIZE);
							strcat(fname, "\\windll.dll");
							if (!DeleteFile(fname))
							{
								strcpy(msg, "Unable to delete file ");
								strcat(msg, fname);
								MessageBox(NULL, msg, "ERROR", 
								MB_OK | MB_ICONWARNING);
							}
						}
					}
					else
						fclose(fp);
				}
			}
		}
	}

	RegCloseKey(hk);
}
