#include "match.h"

/*
 * If str starts with start then return 1, otherwise 0.
 */
int startsWith(char *str, char *start)
{
	return strncmp(str, start, strlen(start)) == 0;
}

/*
 * patt is analysed for multiple characters.
 * index[c][i] gives the current match position for an old match position i
 * and next character c (equivalent to a finite state automaton).
 * UNKNOWN BUG: the values of index[0][...] get set for some reason.
 */
void analyzePattern(int index[CHAR_SIZE][DATA_SIZE], unsigned char *patt)
{
	unsigned int i, j;

	/* initialise index */
	memset(index, 0, CHAR_SIZE * DATA_SIZE * sizeof(int));

	/* if the first character of patt is read the position is 1 */
	for (i = 0; i < DATA_SIZE; i++) {
		index[patt[0]][i] = 1;
	}

	/* if the character is the next character in patt then advance to the
	 * next position.
	 */
	for (i = 0; i < strlen(patt); i++) {
		index[patt[i]][i] = i + 1;
	}

	/* Now comes the complex part.
	 * For each position in patt, consider every possible character being
	 * read in next and update index for each character.
	 */
	for (i = 0; i < strlen(patt); i++) {
		char str[DATA_SIZE] = { '\0' };

		/* get the first i characters of patt */
		strncpy(str, patt, i);

		/* now consider all possible characters read in after str */
		for (j = 0; j < CHAR_SIZE; j++) {
			int k = 0;

			str[i] = j;

			/* find the biggest substring at the end of str that matches
			 * the start of patt.
			 */
			while (str[k] != '\0') {
				char *substr = &str[k]; /* the substring */

				if (startsWith(patt, substr)) {
					/* set the corresponding value in index */
					index[j][i] = strlen(substr);
					break;
				}

				/* next smaller substring */
				k++;
			}
		}
	}

	/* fix the values of index[0][...] */
	memset(index, 0, DATA_SIZE * sizeof(int));
}

/* FAST search routine searches the file fp for the string pointed to by data. */
BOOL searchFile(FILE *fp, char *data)
{
	char c;
	long curr_byte = 0;
	int pos = 0;  /* currently matched position in data */
	int len = strlen(data);
	int matchMatrix[CHAR_SIZE][DATA_SIZE];

	fseek(fp, 0, SEEK_SET);
	analyzePattern(matchMatrix, data);

	c = getc(fp);
	while (!feof(fp)) {

		/* byte read successfully */
		curr_byte++;

		/* get the next match position */
		pos = matchMatrix[(unsigned char) c][pos];

		/* check if the whole pattern has matched */
		if (pos == len) {
			return TRUE;
			/* print out match */
			/*printf("%lx\n", curr_byte - len);
			pos = 0; */
		}

		/* next character */
		c = getc(fp);
	}

	return FALSE;
}
